#include "../../../Platform.h"

#include <WiFi.h>
#include <esp_task_wdt.h>

__NOINIT_ATTR static uint8_t s_rebootCount;
__NOINIT_ATTR static uint16_t s_forceSafeMode;
#define SAFE_MODE_MAGIC 6942

template<>
void
PlatformImpl<HAL_ESP32>::initBootOptions(BootOptions& opts)
{
  opts.resetReason = esp_reset_reason();
  opts.crashCount = s_rebootCount;
  if (opts.resetReason >= 4) { // TODO: These values are defined in
                          // esp32/rom/rtc.h, but not sure if that's included
                          // on platformio builds
    if (opts.crashCount++ >= 3) {
      // Boot into safe mode if the watchdog reset us three times in a row.
      isSafeMode = true;
    }
  } else {
    opts.crashCount = 0;
  }
  s_rebootCount = crashCount;
}

template<>
const char*
PlatformImpl<HAL_ESP32>::name()
{
  return "ESP32";
}

template<>
const char*
PlatformImpl<HAL_ESP32>::version()
{
  return ESP.getSdkVersion();
}

template<>
const char*
PlatformImpl<HAL_ESP32>::deviceID()
{
  uint64_t chipid;
  chipid = ESP.getEfuseMac();
  snprintf(s_deviceID, sizeof(s_deviceID), "%08X", (uint32_t)chipid);
  return s_deviceID;
}

template<>
int
PlatformImpl<HAL_ESP32>::freeRam()
{
  return ESP.getFreeHeap();
}

int printEspLog(const char* fmt, va_list args)
{
  Log.notice(fmt, args);
  return 1;
}

template<>
void
PlatformImpl<HAL_ESP32>::startWatchdog()
{
  esp_task_wdt_init(10, true);
  esp_task_wdt_add(NULL);
  esp_log_set_vprintf(printEspLog);
}

template<>
void
PlatformImpl<HAL_ESP32>::startNTP()
{
  constexpr int dst = 1;
  configTime(0, 3600 * dst, "pool.ntp.org");
}

template<>
void
PlatformImpl<HAL_ESP32>::loop()
{
  esp_task_wdt_reset();
}

template<>
bool
PlatformImpl<HAL_ESP32>::getLocalTime(struct tm* timedata, int timezone)
{
  time_t rawtime;
  memset(&rawtime, 0, sizeof(rawtime));
  time(&rawtime);
  (*timedata) = (*localtime(&rawtime));
  return (timedata->tm_year > (2016-1990));
}

template<>
void
PlatformImpl<HAL_ESP32>::restart()
{
  ESP.restart();
}
